/*---------------------------------------------------------------*/
/* 8-pin SD audio player R0.04                     (C)ChaN, 2010 */
/*---------------------------------------------------------------*/
#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include "diskio.h"
#include "pff.h"

#define FCC(c1,c2,c3,c4)	(((DWORD)c4<<24)+((DWORD)c3<<16)+((WORD)c2<<8)+(BYTE)c1)	/* FourCC */


/*---------------------------------------------------------*/
/* Work Area                                               */
/*---------------------------------------------------------*/

volatile BYTE FifoRi, FifoWi, FifoCt;	/* FIFO controls */

unsigned char Buff1[256];		/* Wave output FIFO */
unsigned char Buff2[256];		/* Wave output FIFO */

FATFS Fs;			/* File system object */
DIR Dir;			/* Directory object */
FILINFO Fno;		/* File information */

WORD rb;			/* Return value. Put this here to avoid bugs of avr-gcc */

/*---------------------------------------------------------*/

static
DWORD load_header (void)	/* 0:Invalid format, 1:I/O error, >1:Number of samples */
{
	DWORD sz;


	if (pf_read(Buff1, 12, &rb)) return 1;	/* Load file header (12 bytes) */

	if (rb != 12 || LD_DWORD(Buff1+8) != FCC('W','A','V','E')) return 0;

	for (;;) {
		pf_read(Buff1, 8, &rb);			/* Get Chunk ID and size */
		if (rb != 8) return 0;
		sz = LD_DWORD(&Buff1[4]);		/* Chunk size */

		switch (LD_DWORD(&Buff1[0])) {	/* FCC */
		case FCC('f','m','t',' ') :					/* 'fmt ' chunk */
			if (sz > 100 || sz < 16) return 0;		/* Check chunk size */
			pf_read(Buff1, sz, &rb);					/* Get content */
			if (rb != sz) return 0;
			if (Buff1[0] != 1) return 0;				/* Check coding type (1) */
			if (Buff1[2] != 1 && Buff1[2] != 2) 		/* Check channels (1/2) */
				return 0;
			
			if (Buff1[14] != 8 && Buff1[14] != 16)	/* Check resolution (8/16) */
				return 0;
			
			OCR0 = (BYTE)(F_CPU/8/LD_WORD(&Buff1[4]))-1;	/* Sampling freq */
			
			break;

		case FCC('d','a','t','a') :				/* 'data' chunk (start to play) */
			return sz;

		case FCC('L','I','S','T') :				/* 'LIST' chunk (skip) */
		case FCC('f','a','c','t') :				/* 'fact' chunk (skip) */
			pf_lseek(Fs.fptr + sz);
			break;

		default :								/* Unknown chunk (error) */
			return 0;
		}
	}

	return 0;
}




volatile uint8_t read1 = 1;

volatile uint16_t counter=0;

volatile uint8_t sync = 0;

volatile uint16_t vu=0;

ISR(TIMER0_COMP_vect)
{
	
	
	if(sync == 1)
	{
		
		if(read1 == 1)
		{
			OCR1A = (unsigned char)Buff1[counter++];
			
			if(counter == 256)
			{
				//PORTC = 0x80;
				read1 = 0;
				counter = 0;
			}
		}
		else if(read1 == 0)
		{
			OCR1A = (unsigned char)Buff2[counter++];
			if(counter == 256)
			{
				//PORTC = 0x40;
				read1 = 1;
				counter = 0;
			}
		}
		vu++;
		
	
		
		//In momentul in care s-a ajuns la 500 de iteratii
		//aprindem ledurile in functie de amplitudinea 			//semnalului citit

		//Are loc o impartire in 5 subintervale a 				//intervalului de amplitudini 
		if(vu == 500)
		{
			PORTC = 0x00;
			if(OCR1A > 130 && OCR1A < 145)
				PORTC = 0b10000001;
			else if(OCR1A < 160)
			PORTC = 0b10000011;
			else if(OCR1A < 180)
			PORTC = 0b11000011;
			else if(OCR1A < 200)
			PORTC = 0b11100111;
			else if(OCR1A < 220)
			PORTC = 0b11111111;
			
			//Resetare contor
			vu = 0;
		}
		
	}
}

void Timer0_init(void)
{
	TCCR0 = _BV(WGM01)|_BV(CS01);//CTC, FCPU/8
	TIMSK = _BV(OCIE0);
}

void Timer1_init(void)
{

	TCCR1A = _BV(COM1A1)|_BV(COM1A0)|_BV(WGM10);
	TCCR1B = _BV(WGM12)|_BV(CS10);
}


static UINT play (const char *fn)
{
	DWORD sz;
	FRESULT res;
	unsigned char read2;

	
	uint16_t size = 256;
	

	if ((res = pf_open(fn)) == FR_OK) {
	
		
		
		Timer0_init();
		Timer1_init();
		
		sz = load_header();			/* Load file header */
		if (sz < 256) return (UINT)sz;
		
		
		FifoCt = 0; FifoRi = 0; FifoWi = 0;		/* Reset FIFO */
		pf_read(Buff1, 512 - (Fs.fptr % 512), &rb);	/* Snip sector unaligned part */
		sz -= rb;

			
		res = pf_read(Buff1, size, &rb);
		if (res != FR_OK) { PORTC = 0x08; while (FifoCt) ; return res; }
		
		sync = 1;
				
		do {
				
			read2 = read1;
			
			if(read1 == 1)	
				res = pf_read(Buff2, size, &rb);
			else
				res = pf_read(Buff1, size, &rb);
				
			if (res != FR_OK) { PORTC = 0x08; break; }
			
			while(read2 == read1);
			
			if(!(PINB&(1<<PB1))) 
			{
				sync = 0;
				_delay_ms(500); 
				break;
			}
			
		
		} while (rb == size);	/* Repeat until all data read */
		
		sync = 0;
	}

	while (FifoCt) ;			/* Wait for FIFO empty */
	

	return res;
}





/*-----------------------------------------------------------------------*/
/* Main                                                                  */


int main (void)
{
	int i, j;
	int schimba = 1, count = 0;
	
	DDRD |= (1<<PD5)|(1<<PD6);
	DDRB &= ~(1<<PB1);
	PORTB |= (1<<PB1);
	PORTD &= ~((1<<PD5)|(1<<PD6));
	DDRC = 0xff;
	PORTC = 0x00;
	DDRA = 0xff;
	PORTA = 0xff;
	PORTA = 0x00;
	PORTC = 0xff;

	

	//Animatii scriptate leduri
	for(j=0; j<5; j++)
	{
		for(i=0; i<4; i++){
			PORTA |= (1<<(3-i)) | (1<<(4 + i));
			_delay_ms(50);
		}
		for(i=0; i<4; i++){
			PORTA &= ~(1<<(3-i)) & ~(1<<(4 + i));
			_delay_ms(50);
		}
		_delay_ms(100);
	}
	PORTC = 0x00;
	PORTA = 0x00;	
	_delay_ms(1000);


	i = 0;
	schimba = 1;
	count = 0;
	while(count < 20){
		schimba = 1-schimba;
		if(schimba == 0){
			for(j = 0; j < 8; j++)
			{
				i = 0;
				while(i < 7)
				{
					PORTA = 0x00;
					PORTA |= (1<<j);
					PORTC = 0b00000110;
					_delay_ms(5);
					PORTA = 0x00;
					PORTA |= (1<<(7-j));
					PORTC = 0b10010000;
					_delay_ms(5);
					i++;
				}
			}
		}
		else
		{
			for(j = 7; j >= 0; j--)
			{
				i = 0;
				while(i < 7)
				{
					PORTA = 0x00;
					PORTA |= (1<<j);
					PORTC = 0b00001001;
					_delay_ms(5);
					PORTA = 0x00;
					PORTA |= (1<<(7-j));
					PORTC = 0b01100000;
					_delay_ms(5);
					i++;
				}
			}
		}
		count++;
	}	

	PORTC = 0x00;
	PORTA = 0x00;	
	_delay_ms(1000);

	count = 0;
	while(count < 7){
		j = 200;
		PORTC = 0x0f;
		for(i=0; i<8; i++)
		{
			PORTA = 0x00;
			PORTA |= (1<<i);
			_delay_ms(j);
			j /= 1.2;
		}
		
		PORTC = 0xf0;
		
		for(i=0; i<8; i++)
		{
			PORTA = 0x00;
			PORTA |= (1<<i);
			_delay_ms(j);
			j *= 1.2;
		}
		count++;
	}
	count = 0;
	while(count < 5){
		for(i=0; i<4; i++){
		
			PORTC |= (1<<i);
			for(j=0; j<8; j++){
				PORTA |= (1<<j);
				_delay_ms(50);
				PORTA = 0x00;
			}
			
			PORTC = 0x00;
			PORTC |= (1<<(7-i));
			for(j=0; j<8; j++){
				PORTA |= (1<<j);
				_delay_ms(50);
				PORTA = 0x00;
			}
			
			PORTC = 0x00;
		}
		count++;
	}


	PORTC = 0x00;
	PORTA = 0x00;	
	_delay_ms(1000);


	//sei();
	

	//Citire de pe card si redare
	for (;;) {
		_delay_ms(1000);					/* Delay 1000ms */
		if (pf_mount(&Fs)) 
		{
			_delay_ms(1000);
			for(i = 0; i < 50; i++)
			{
				if(j == 0) PORTC = 0b00110011;
				else 	   PORTC = 0b11001100;
				PORTA |= (1<<PA0) | (1<<PA7);
				_delay_ms(2);
				PORTA = 0x00;
			}


			continue;	/* Initialize FS */
		}
	 	PORTD |= (1<<PD7);
		for (;;) {
			if (pf_opendir(&Dir, "")) break;	/* Open sound file directory (root dir) */
			while (!pf_readdir(&Dir, &Fno) && Fno.fname[0]) {	/* Play all wav files in the dir */
				if (!(Fno.fattrib & (AM_DIR|AM_HID)) && strstr(Fno.fname, ".WAV")) {
					if (play(Fno.fname)) break;
				}
			}
		}
	}
}



